require "kintone_notifier/version"
require 'kintone_notifier/bot'
require 'kintone'

module KintoneNotifier

  def self.included(base)
    base.extend(ClassMethods)
    base.class_eval do
      include KintoneNotifier::InstanceMethods
      if Rails.env.production?
        rescue_from Exception, :with => :render_500
        rescue_from ActiveRecord::RecordNotFound, :with => :render_404
        rescue_from ActionController::RoutingError, :with => :render_404
        rescue_from ActionView::MissingTemplate, :with => :render_404
      end
      helper_method :request_env_string

    end
  end

  # configuration (required)
  #   url       : includes ".cybozu.com"
  #   user      : kintone user name
  #   password  : kintone user password
  #   app_id    : kintone application id
  # examples
  # #in your config/initializers/
  #  KintoneNotifier::Bot.config = {
  #    :url => "xxxxx.cybozu.com",
  #    :user  => "your-user-name",
  #    :password => "your-password",
  #    :app_id => 1
  #  }

  module ClassMethods

  end
  module InstanceMethods
    protected
    def render_404
      render :template => 'errors/404', :layout => false, :status => 404
    end

    def render_500(exception=nil)
      # render :file => "#{Rails.root}/public/500.html", :status => 500, :layout => false, :content_type => 'text/html'

      if exception.present?
        bot = KintoneNotifier::Bot.new
        notice = {
          "title"           =>  compose_exception_title(exception),
          "url"             =>  request.url,
          "http_method"     =>  request.request_method,
          "remote_ip"       =>  request.remote_ip,
          "parameters"      =>  request.filtered_parameters.inspect,
          "exception_class_name"  =>  exception.class.name,
          "description"     =>  exception.message,
          "backtrace"       =>  exception.backtrace.join("\r\n"),
          "session"         =>  session.inspect,
          "environment"     =>  request_env_string,
          "user_agent"      =>  request.headers["HTTP_USER_AGENT"]
        }
        bot.tweet(notice)
      end
      render :file => 'errors/500', :layout => false, :status => 500
    end

    def request_env_string
      request.env.map{|k,v|"#{k}:#{v}"}.join("\r\n")
    end

    def compose_exception_title(exception)
      str = ""
      str << "#{self.controller_name}##{self.action_name}"
      str << " (#{exception.class})"
      str << " #{exception.message.inspect}"
      str.length > 120 ? str[0...120] + "..." : str
    end

    def compose_exception_description(exception)
      [].tap do |vars|
        vars << "======================================"
        vars << "Base Message"
        vars << "======================================"
        vars << exception.message
        vars << ""
        vars << ""
        vars << "======================================"
        vars << "BackTrace"
        vars << "======================================"
        vars << exception.backtrace

      end.join("\r\n")
    end
  end
end
